/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.empfang.ftpempfang;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.StandardCopyOption;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bsvrz.kex.lmstic3.vew.LMSFTPListener;
import de.bsvrz.kex.lmstic3.vew.LMSKommunikationsstatus;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.LMSProperties;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Modul FTP-Empfang.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */
public class FTPEmpfang extends Thread {
	/**
	 * Filter zur Dateiauswahl.
	 */
	class XmlFilenameFilter implements FilenameFilter {
		@Override
		public boolean accept(final File f, final String s) {
			return s.toLowerCase().endsWith(".xml");
		}
	}

	/**
	 * Logger f&uuml;r Debug-Ausgaben.
	 */
	private final Debug debug = Debug.getLogger();

	/**
	 * Zyklus, mit dem das Eingangsverzeichnis auf neue Dateien
	 * &uuml;berpr&uuml;ft wird.
	 */
	private final int zyklus = 10000;

	/**
	 * Die aktuellen Parameter.
	 */
	private LMSParameter parameter;

	/**
	 * Liste der angemeldeten Listener.
	 */
	private final List<LMSFTPListener> listenerList = new ArrayList<>();

	/**
	 * Der Kommunikationsstatus mit der Landesmeldestelle.
	 */
	private LMSKommunikationsstatus kommunikationsStatus = LMSKommunikationsstatus.GESTOERT;

	/**
	 * HashSet der bereits empfangenen Dateien.
	 */
	private Map<File, Long> dateimap = new HashMap<>();

	/**
	 * Zeitpunkt des letzten Empfanges einer Datei.
	 */
	private long letzterEmpfang;

	/**
	 * Run-Flag.
	 */
	private boolean run = true;

	/**
	 * Erzeugt eine Instanz und startet diese als Thread.
	 *
	 * @param parameter
	 *            Der Parametersatz
	 */
	public FTPEmpfang(final LMSParameter parameter) {
		this.parameter = parameter;
		this.setName(this.getClass().getSimpleName());
		// selbst starten
		start();
	}

	/**
	 * F&uuml;gt einen Listener hinzu.
	 *
	 * @param listner
	 *            neuer Listener
	 */
	public void addListener(final LMSFTPListener listner) {
		listenerList.add(listner);
	}

	/**
	 * &Uuml;berpr&uuml;ft das Eingangsverzeichnis auf neue empfangene Dateien.
	 */
	private void findeNeueDateien() {
		final Map<File, Long> dateimapneu = new HashMap<>();

		final File dir = new File(parameter.getEingangsverzeichnis());

		final File[] dateien = dir.listFiles(new XmlFilenameFilter());

		List<File> neueDateien = new ArrayList<>();

		if (dateien != null) {
			for (final File datei : dateien) {
				if (datei.getName().equalsIgnoreCase(LMSProperties.KEEP_ALIVE_DATEINAME)) {
					debug.info("KeepAlive-Datei empfangen");
					letzterEmpfang = System.currentTimeMillis();
					setzeKommunikationsStatus(LMSKommunikationsstatus.NICHT_GESTOERT);
					datei.delete();
					continue;
				}

				boolean isNeu = false;

				// Test, ob immer die gleiche Datei überschrieben wird
				if (dateimap.containsKey(datei)) {
					final Long fileTime = dateimap.get(datei);

					if (fileTime != datei.lastModified()) {
						isNeu = true;
					}
				} else {
					isNeu = true;
				}

				if (isNeu) {
					dateimapneu.put(datei, datei.lastModified());
					neueDateien.add(datei);
				}
			}
		}

		dateimap = dateimapneu;

		neueDateien = checkFileReady(neueDateien);

		neueDateien = handleOverrideMode(neueDateien);

		if (neueDateien.size() > 0) {
			debug.fine(neueDateien.size() + " neue Datei(en) empfangen (" + neueDateien + ")");
			setzeKommunikationsStatus(LMSKommunikationsstatus.NICHT_GESTOERT);
			letzterEmpfang = System.currentTimeMillis();
			informiereListenerDateien(neueDateien);
		}
	}

	/**
	 * Prüft, ob der Transfer der Dateien beendet ist.
	 * <p>
	 * Es wird geprüft, ob sich innerhalb einer Sekunde die Größe ändert.
	 * </p>
	 * 
	 * @param neueDateien
	 *            neue Dateien
	 * @return Liste von Dateien
	 */
	private List<File> checkFileReady(final List<File> neueDateien) {
		final List<File> result = new ArrayList<>();

		final Map<File, Long> sizeMap = new HashMap<>();

		for (File f : neueDateien) {
			final long length = f.length();
			sizeMap.put(f, length);
		}

		try {
			Thread.sleep(1000);
		} catch (final InterruptedException e) {
		}

		for (File f : neueDateien) {
			final long length = f.length();
			if (sizeMap.get(f) == length) {
				result.add(f);
			}
		}

		return result;
	}

	private List<File> handleOverrideMode(final List<File> dateien) {
		final List<File> neueDateien = new ArrayList<>();

		final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMdd_HHmmss");
		final String ts = simpleDateFormat.format(new Date(System.currentTimeMillis()));

		for (File f : dateien) {
			final String destname = f.getAbsolutePath() + "_" + ts;
			final File dest = new File(destname);
			try {
				Files.move(f.toPath(), dest.toPath(), StandardCopyOption.ATOMIC_MOVE);
				neueDateien.add(dest);
			} catch (final IOException e) {
				debug.error(e.getMessage());
			}
		}

		return neueDateien;
	}

	/**
	 * Informiert die Listener &uuml;ber neue Dateien.
	 *
	 * @param dateien
	 *            Liste der neuen Dateien
	 */
	void informiereListenerDateien(final List<File> dateien) {
		for (final LMSFTPListener listener : listenerList) {
			listener.empfangen(dateien);
		}
	}

	/**
	 * Informiert die Listener &uuml;ber den Kommunikationsstatus.
	 */
	void informiereListenerStatus() {
		for (final LMSFTPListener listener : listenerList) {
			listener.kommunikationsStatus(kommunikationsStatus);
		}
	}

	@Override
	public void run() {
		try {
			while (run) {
				Thread.sleep(zyklus);
				findeNeueDateien();

				// ueberpruefe Kommunikationsstatus
				if ((System.currentTimeMillis() - letzterEmpfang) > parameter.getTestIntervallKommunikation()) {
					debug.warning("Der Kommunikationsstatus ist gestört");
					setzeKommunikationsStatus(LMSKommunikationsstatus.GESTOERT);
					letzterEmpfang = System.currentTimeMillis();
				}
			}
		} catch (final InterruptedException ie) {
			/**
			 *
			 */
		} catch (final Exception e) {
			debug.error(e.getMessage());
		}

	}

	/**
	 * Setzt den internen Kommunikationstatus und benachrichtigt die Listener.
	 *
	 * @param neuerStatus
	 *            neuer Kommunikationsstatus
	 */
	private void setzeKommunikationsStatus(final LMSKommunikationsstatus neuerStatus) {
		final boolean notify = neuerStatus != kommunikationsStatus;
		kommunikationsStatus = neuerStatus;
		if (notify) {
			informiereListenerStatus();
		}
	}

	/**
	 * Stoppt den Thread.
	 */
	public void stopp() {
		this.run = false;
		this.interrupt();
	}
}
