/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic;

import java.util.concurrent.TimeUnit;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.lmstic3.vew.DavSendeAnmeldung;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.ExterneMeldung;
import de.bsvrz.kex.lmstic3.vew.daten.ExterneMeldungsDaten;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * SWE-Interne Repr&auml;sentation einer TIC3-Meldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */

public class TIC3Meldung implements ClientSenderInterface, ExterneMeldung {

	private static final String DAVTYP = "typ.tmcTic3infoMeldung";

	/**
	 * XML-Tag TIC3-Meldung.
	 */
	public static final String XML_ELEMENT_MELDUNG = "TrafficAndTravelEvent";

	/**
	 * DebugLogger f&uuml;r Debug-Ausgaben.
	 */
	private static final Debug LOGGER = Debug.getLogger();

	private static final String ATG = "atg.tmcTic3infoMeldung";

	private static final String ASP = "asp.lmsEmpfangen";

	/**
	 * Das zugeh&ouml;rige Systemobjekt.
	 */
	private SystemObject systemObject;

	/**
	 * Flag, ob die Datenanmeldung beim DaV erfolgt ist.
	 */
	private boolean istAngemeldet;

	/**
	 * Synchronisationsobjekt f&uuml;r die Datenanmeldung.
	 */
	private final Object anmeldungsLock = new Object();

	/** Wartezeit auf erfolgreiche Anmeldebest&auml;tigung. */
	private final long warteZeitAnmeldung = TimeUnit.SECONDS.toMillis(20);

	/** die Daten der Meldung. */
	private TIC3MeldungsDaten meldungsDaten;

	/** Datenspezifikation f&uuml;r die empfangenen Daten. */
	private DavSendeAnmeldung dataEmpfangen;

	/**
	 * Erzeugt eine neue TIC3-Meldung aus einem Systemobjekt.
	 *
	 * @param object
	 *            Systemobjekt
	 */
	public TIC3Meldung(final SystemObject object) {
		this(object, new TIC3MeldungsDaten());
	}

	/**
	 * Erzeugt eine neue TIC-Meldung aus einem Systemobjekt und Meldungsdaten.
	 *
	 * @param object
	 *            Systemobjekt
	 * @param daten
	 *            Meldungsdaten
	 */
	public TIC3Meldung(final SystemObject object, final TIC3MeldungsDaten daten) {
		systemObject = object;
		meldungsDaten = daten;
	}

	/**
	 * Erzeugt eine neue TIC-Meldung aus einem Systemobjekt und Meldungsdaten.
	 *
	 * @param daten
	 *            Meldungsdaten
	 */
	public TIC3Meldung(final TIC3MeldungsDaten daten) {
		meldungsDaten = daten;
	}

	@Override
	public void anmeldenDav(final ClientDavInterface dav) throws TIC3Exception {

		if (istAngemeldet) {
			return;
		}

		final DataModel model = dav.getDataModel();

		dataEmpfangen = new DavSendeAnmeldung(model.getAttributeGroup(ATG), model.getAspect(ASP));

		try {
			TIC3Meldung.LOGGER.fine("beginne Datenanmeldung Empfangsdaten: " + this);

			synchronized (anmeldungsLock) {
				dav.subscribeSender(this, systemObject, dataEmpfangen, SenderRole.source());

				// warte auf Anmeldebestätigung
				TIC3Meldung.LOGGER.fine("beginne Wartezeit auf Bestätigung der Datenanmeldung: " + this);

				try {
					anmeldungsLock.wait(warteZeitAnmeldung);
				} catch (final InterruptedException e) {
					// wird nicht behandelt
				}
			}

			TIC3Meldung.LOGGER.fine("Ende Warten auf Bestätigung der Datenanmeldung: " + this);

			if (!istAngemeldet) {
				throw new TIC3Exception("Die Meldung " + systemObject.getPid()
						+ " konnte nicht erfolgreich am DaV angemeldet werden" + " (es erfolgte innerhalb von "
						+ (warteZeitAnmeldung / 1000) + "s keine Rückmeldung über die Sendeanmeldung)");
			}

		} catch (final OneSubscriptionPerSendData e) {
			throw new TIC3Exception("Fehler bei DAV Anmeldung: Objekt: " + systemObject + " (" + e.getMessage() + ")");
		}
	}

	/**
	 * Datenverteilerabmeldung f&uuml;r empfangene Meldungen.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	@Override
	public void abmeldenDavEmpfang(final ClientDavInterface dav) throws TIC3Exception {
		dav.unsubscribeSender(this, systemObject, dataEmpfangen);
	}

	/**
	 * Erzeugt eine PID aus den Meldungsdaten.
	 *
	 * @return PID
	 */
	@Override
	public String bildePid() {
		return meldungsDaten.bildePid();
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		boolean anmeldungOK = false;

		TIC3Meldung.LOGGER.finer("dataRequest: " + this + ", DataDescription: " + dataDescription);

		if (dataDescription == dataEmpfangen) {
			// die Daten werden auch gesendet, wenn kein Empfänger angemeldet
			// ist
			dataEmpfangen.setzeSendenOK(
					(state == ClientSenderInterface.START_SENDING) || (state == ClientSenderInterface.STOP_SENDING));
			anmeldungOK = dataEmpfangen.sendenOK();
		}

		if (anmeldungOK && !istAngemeldet) {
			istAngemeldet = true;
			TIC3Meldung.LOGGER
					.finer("dataRequest: " + this + " Lock-Freigabe: " + istAngemeldet + ", " + anmeldungsLock);
			if (anmeldungsLock != null) {
				synchronized (anmeldungsLock) {
					anmeldungsLock.notify();
				}
			}
		}
	}

	/**
	 * Vervollst&auml;ndigt die Daten der Meldung.
	 *
	 * InfoGUID (IID): Identifikation, aktuelle Systemzeit (Millisekunden)
	 * IDOrganisation (ORG): aus RDS-Meldung oder wenn nicht gesetzt aus
	 * Parameter atg.rdsKommunikationsParameter.EigeneID. IDNumber (NUM):
	 * inkrementierte laufende Nummer aus
	 * atg.rdsKommunikationsParameter.LaufendeNummer ReminderTime (REM) wird
	 * gebildet aus aktueller Zeit + Wert des Attributes
	 * atg.rdsKommunikationsParameter.SendeWiederholung
	 *
	 * @param parameter
	 *            LMS-Parameter
	 */
	public void datenvervollstaendigen(final LMSParameter parameter) {
		if ((meldungsDaten.getDatenKennung() == null) || (meldungsDaten.getDatenKennung().length() == 0)) {
			meldungsDaten.setzeIdentifikation(String.valueOf(System.currentTimeMillis()));
		}

		if ((meldungsDaten.getDatenProduzent() == null) || (meldungsDaten.getDatenProduzent().length() == 0)) {
			meldungsDaten.setIDOrganisation(parameter.getEigeneID());
		}

		meldungsDaten.setIDNumber(String.valueOf(parameter.getLaufendeNummer()));
	}

	/**
	 * Erzeugt das XML-Element f&uuml;r die Meldung.
	 *
	 * @param doc
	 *            Dokument
	 * @param root
	 *            Wurzelelement
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public void erzeugeDocumentElement(final Document doc, final Element root) throws TIC3Exception {
		meldungsDaten.erzeugeDocumentElement(doc, root);
	}

	/**
	 * Gibt die Daten der Meldung zur&uuml;ck.
	 *
	 * @return Daten der Meldung
	 */
	@Override
	public TIC3MeldungsDaten getMeldungsDaten() {
		return meldungsDaten;
	}

	/**
	 * Gibt die PID des zugeh&ouml;rigen Systemobjektes zur&uuml;ck.
	 *
	 * @return PID
	 */
	public String getPID() {
		return meldungsDaten.getID();
	}

	/**
	 * Gibt das zugeh&ouml;rige Systemobjekt zur&uuml;ck.
	 *
	 * @return zugeh&ouml;riges Systemobjekt
	 */
	@Override
	public SystemObject getSystemObject() {
		return systemObject;
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return true;
	}

	/**
	 * Publiziert die Daten zum DaV.
	 *
	 * @param dav
	 *            Datenverteilerverbindung
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	@Override
	public void publiziereDav(final ClientDavInterface dav, final LMSParameter lmsParameter) throws TIC3Exception {
		try {
			final Data data = dav.createData(dataEmpfangen.getAttributeGroup());
			meldungsDaten.publizieren(data, lmsParameter);
			final ResultData meldungsdaten = new ResultData(systemObject, dataEmpfangen, System.currentTimeMillis(),
					data);
			if (dataEmpfangen.sendenOK()) {
				TIC3Meldung.LOGGER.finest("Sende Daten (empfangen) zum DAV: " + meldungsdaten);
				dav.sendData(meldungsdaten);
			}
		} catch (final DataNotSubscribedException e) {
			TIC3Meldung.LOGGER.error(e.getMessage());
		} catch (final SendSubscriptionNotConfirmed e) {
			TIC3Meldung.LOGGER.error(e.getMessage());
		}
	}

	/**
	 * Setzt die Daten der Meldung.
	 *
	 * @param meldungsdaten
	 *            Daten der Meldung
	 */
	public void setMeldungsDaten(final TIC3MeldungsDaten meldungsdaten) {
		this.meldungsDaten = meldungsdaten;
	}

	@Override
	public String toString() {
		return getPID();
	}

	@Override
	public void setSystemObject(final SystemObject object) {
		systemObject = object;
	}

	@Override
	public String getDavTyp() {
		return DAVTYP;
	}

	@Override
	public String getID() {
		return getMeldungsDaten().getID();
	}

	@Override
	public void setMeldungsDaten(final ExterneMeldungsDaten daten) {
		this.meldungsDaten = (TIC3MeldungsDaten) daten;
	}

}
