/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.ExterneMeldungsDaten;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Event;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Koordinaten;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Location;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Prioritaet;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Prioritaet.TIC3PrioritaetEnum;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Quelle;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Status;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Status.TIC3StatusEnum;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3TMCEvent;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3TMCLocation;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Text;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3TmcEventCodeTyp.TIC3TmcEventCodeTypEN;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Umleitung;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3ZeitZone;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Zeitplan;
import de.bsvrz.kex.lmstic3.vew.daten.tic.elemente.TIC3Zeitstempel;
import de.bsvrz.kex.lmstic3.vew.daten.tmc.TMCEvent;
import de.bsvrz.kex.lmstic3.vew.daten.tmc.TMCEvent.TMCEventType;
import de.bsvrz.kex.lmstic3.vew.daten.tmc.TMCKoordinate;
import de.bsvrz.kex.lmstic3.vew.daten.tmc.TMCLocation;
import de.bsvrz.kex.lmstic3.vew.daten.tmc.TMCPrioritaet.TMCPrioritaetEnum;
import de.bsvrz.kex.lmstic3.vew.daten.tmc.TMCRichtung.TMCRichtungEnum;
import de.bsvrz.kex.lmstic3.vew.daten.tmc.TMCStatus.TMCStatusEnum;

/**
 * Datensatz einer TIC3-Meldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */

public class TIC3MeldungsDaten extends AbstractTIC3InfoElement implements ExterneMeldungsDaten {

	private final TIC3InfoAttribut beschreibung = new TIC3InfoAttribut("description", null);

	/**
	 * Identifikator.
	 * <p>
	 * TIC3 Info XML Element: TicId (TID), Globaler eindeutiger Identifikator
	 * der Information.
	 * </p>
	 */
	private final TIC3Text tic3Id = new TIC3Text("TicId", "TIC3TicId");

	/**
	 * Datenerzeugende Organisation.
	 * <p>
	 * TIC3 Info XML Element: DataProducer (DPR), Organisation, welche die Daten
	 * erzeugt hat.
	 * </p>
	 */
	private final TIC3Text tic3Datenproduzent = new TIC3Text("DataProducer", "TIC3Datenproduzent");

	/**
	 * Identifikation.
	 * <p>
	 * TIC3 Info XML Element: DataIdentifier (DID), eindeutige Identifikation.
	 * </p>
	 */
	private final TIC3Text tic3Datenkennung = new TIC3Text("DataIdentifier", "TIC3Datenkennung");

	/**
	 * Beschreibung des Ereignisses.
	 * <p>
	 * TIC3 Info XML Element: Description (DES), Beschreibung des Ereignisses.
	 * </p>
	 */
	private final TIC3Text tic3Beschreibung = new TIC3Text("Description", "TIC3Beschreibung");

	/**
	 * Lokationsinformationen.
	 * <p>
	 * TIC3 Info XML Element: Location (LOC), Information about a location
	 * specified by one or more location referencing methods such as TIC3
	 * location, geographical position, or TMC location.
	 * </p>
	 */
	private final TIC3Location tic3Location = new TIC3Location("Location", "TIC3Location");

	/**
	 * Ereignisdaten.
	 * <p>
	 * TIC3 Info XML Element: Event (EVT), Event codes with additional
	 * information for different event coding methods, such as TIC3, TMC, or SAE
	 * RDS.
	 * </p>
	 */
	private final TIC3Event tic3Event = new TIC3Event();

	/**
	 * Dauer.
	 * <p>
	 * TIC3 Info XML Element: Duration (DUR), A schedule specifying the duration
	 * of the traffic and travel event. The schedule is specified by the overall
	 * local begin and end times which can then be limited to recurring days and
	 * times. The time zone specified in element "Time zone" is used to convert
	 * between local and UTC times.
	 * </p>
	 */
	private final TIC3Zeitplan dauer = new TIC3Zeitplan("Duration", "TIC3Dauer");

	/**
	 * Umleitungen.
	 * <p>
	 * TIC3 Info XML Element: DiversionRoute (DRO), A list of diversion routes
	 * and the vehicle types to which the diversion applies.
	 * </p>
	 */
	private static final String DAV_ATTR_UMLEITUNGEN = "TIC3Umleitungen";
	private List<TIC3Umleitung> umleitungen = new ArrayList<>();

	/**
	 * Nicht kodierbare Information.
	 * <p>
	 * TIC3 Info XML Element: AdditionalNonCodeableInformation (ANI), A
	 * culture-dependent free text providing additional information which cannot
	 * be coded in other data elements.
	 * </p>
	 */
	private final TIC3Text nichtKodierbareInformation = new TIC3Text("AdditionalNonCodeableInformation",
			"TIC3NichtKodierbareInformation");

	/**
	 * Priorität.
	 * <p>
	 * TIC3 Info XML Element: Urgency (URG), The urgency such as normal, urgent,
	 * or extremely urgent.
	 * </p>
	 */
	private final TIC3Prioritaet prioritaet = new TIC3Prioritaet("Urgency", "TIC3Prioritaet");

	/**
	 * Lokationsinformationen der Gegenrichtung.
	 * <p>
	 * TIC Info XML Element: OppositeDirectionLocation (OPP), When the condition
	 * or event is in both directions, information about the location which
	 * corresponds to the opposite direction of traffic, specified by one or
	 * more location referencing methods such as TIC3 location, geographical
	 * position, or TMC location.
	 * </p>
	 */
	private final TIC3Location tic3LocationGegenrichtung = new TIC3Location("OppositeDirectionLocation",
			"TIC3LocationGegenrichtung");

	/**
	 * Quellen.
	 * <p>
	 * TIC3 Info XML Element: Source (SRC), A list of references to and
	 * information about the source data used to create the data contained in
	 * this object.
	 * </p>
	 */
	private static final String DAV_ATTR_QUELLEN = "TIC3Quellen";
	private List<TIC3Quelle> quellen = new ArrayList<>();

	/**
	 * Name.
	 * <p>
	 * TIC3 Info XML Element: Name (NAM), The name of the object, in one or more
	 * languages configured on the system.
	 * </p>
	 */
	private final TIC3Text name = new TIC3Text("Name", "TIC3Name");

	/**
	 * Erzeugt durch Prozess.
	 * <p>
	 * TIC3 Info XML Element: CreatedByProcess (CBP), The manually create data
	 * process, specifying the process in which the data have been created. A
	 * reference to "ManuallyCreateDataProcess".
	 * </p>
	 */
	private final TIC3Text erzeugtDurchProzess = new TIC3Text("CreatedByProcess", "TIC3ErzeugtDurchProzess");

	/**
	 * TIC3 Datenerzeugungszeit.
	 * <p>
	 * TIC Info XML Element: CreateTime (CRT), The date and time when the data
	 * contained in this object version was created.
	 * </p>
	 */
	private final TIC3Zeitstempel datenerzeugungszeit = new TIC3Zeitstempel("CreateTime", "TIC3Datenerzeugungszeit");

	/**
	 * TIC3 Datenersterzeugungszeit.
	 * <p>
	 * TIC Info XML Element: FirstCreateTime (FCT), The date and time when the
	 * data contained in the first object version was created.
	 * </p>
	 */
	private final TIC3Zeitstempel datenersterzeugungszeit = new TIC3Zeitstempel("FirstCreateTime",
			"TIC3Datenersterzeugungszeit");

	/**
	 * TIC3 Zeitzone.
	 * <p>
	 * TIC3 Info XML Element: TimeZone (TZO), The time zone as defined for
	 * Microsoft operating systems, described by a name and offset to
	 * Coordinated Universal Time (UTC), and used for time data which is
	 * provided in the local time and which has been adjusted to local Daylight
	 * Savings Time (DST).
	 * </p>
	 */
	private final TIC3ZeitZone zeitZone = new TIC3ZeitZone("TimeZone", "TIC3ZeitZone");

	// TODO
	// <!-- <TimeZone name="Zeitzone" value="(UTC+01:00) Amsterdam, Berlin,
	// Bern,
	// Rom, Stockholm, Wien">4</TimeZone> -->
	// <!-- <ObjectCategory value="Dauermeldung">TIC-LMS-BW/1</ObjectCategory>
	// <ManuallySpecifiedData>ObjectCategory</ManuallySpecifiedData> -->

	/**
	 * Lokationsinformationen der kreuzenden Lokationen.
	 * <p>
	 * TIC Info XML Element: CrossLocation (CLO), One or more roads crossing the
	 * roads specified by the location, which are also affected by the event.
	 * Each item in the list contains information about a crossing road
	 * specified by one or more location referencing methods such as TIC3
	 * location, geographical position, or TMC location.
	 * </p>
	 */
	private static final String XML_ELEMENT_CROSSLOCATIONS = "CrossLocation";
	private static final String DAV_ATTR_CROSSLOCATIONS = "TIC3KreuzungsLokationen";
	private List<TIC3Location> kreuzungsLokationen = new ArrayList<>();

	/**
	 * Keine weiteren Aktualisierungen.
	 * <p>
	 * TIC3 Info XML Element: NoFurtherUpdates (NUP), Specifies whether or not
	 * it is likely that no further identifiable updates will be received.
	 * </p>
	 */
	private final TIC3Boolean weitereUpdates = new TIC3Boolean("NoFurtherUpdates", "TIC3KeineWeiterenAktualisierungen");

	/**
	 * TIC3 Objektversionserzeugungszeit.
	 * <p>
	 * TIC Info XML Element: ObjectVersionBeginTime (OVB), The date and time
	 * when this version of the object was created.
	 * </p>
	 */
	private final TIC3Zeitstempel objektErzeugungszeit = new TIC3Zeitstempel("ObjectVersionBeginTime",
			"TIC3Objektversionserzeugungszeit");

	/**
	 * TIC3 Objektversionsendezeit.
	 * <p>
	 * TIC Info XML Element: ObjectVersionEndTime (OVE), The date and time when
	 * this version of the object ended. The end time is assigned when a new
	 * object version is created or the current version is deleted.
	 * </p>
	 */
	private final TIC3Zeitstempel objektEndezeit = new TIC3Zeitstempel("ObjectVersionEndTime",
			"TIC3Objektversionsendeszeit");

	/**
	 * TIC3 Objektversionsstatus.
	 * <p>
	 * TIC Info XML Element: ObjectVersionStatus (OVS), The type of the object
	 * version, specifying whether the object is new, changed, or deleted.
	 * </p>
	 */
	private final TIC3Status tic3Status = new TIC3Status("ObjectVersionStatus", "TIC3Status");

	/**
	 * TIC3 Speicherzeit.
	 * <p>
	 * TIC Info XML Element: StoreTime (STT), The date and time when this
	 * version of the data was stored.
	 * </p>
	 */
	private final TIC3Zeitstempel speicherZeit = new TIC3Zeitstempel("StoreTime", "TIC3Speicherzeit");

	/**
	 * Erzeugt durch Benutzer.
	 * <p>
	 * TIC3 Info XML Element: CreatedByUser (CBU), The user who created this
	 * version of the data. A reference to "User".
	 * </p>
	 * <p>
	 * Das sieht so aus:<br>
	 * <code>&lt;CreatedByUser name="Erzeugt durch Benutzer" value="(LMS) Horst Müller"&gt;TIC-LMS-BW/40&lt;/CreatedByUser&gt;</code><br>
	 * Wir benutzen den Wert aus dem 'value'-Attribut.
	 * </p>
	 */
	private final TIC3Text erzeugtDurchBenutzer = new TIC3Text("CreatedByUser", "TIC3ErzeugtDurchBenutzer", "value");

	/**
	 * Erzeugt durch Team.
	 * <p>
	 * TIC3 Info XML Element: CreatedByTeam (CBT), The team which created this
	 * version of the data. A reference to "Team".
	 * </p>
	 * <p>
	 * Das sieht so aus:<br>
	 * <code>&lt;CreatedByTeam name="Erzeugt durch Team" value="LMS - Admin"&gt;TIC-LMS-BW/20&lt;/CreatedByTeam&gt;</code><br>
	 * Wir benutzen den Wert aus dem 'value'-Attribut.
	 * </p>
	 */
	private final TIC3Text erzeugtDurchTeam = new TIC3Text("CreatedByTeam", "TIC3ErzeugtDurchTeam", "value");

	/**
	 * Erzeugt durch TIC Server Komponente.
	 * <p>
	 * TIC3 Info XML Element: CreatedByComponent (CBC), The TIC Server component
	 * which created this version of the data. A reference to "Component".
	 * </p>
	 * <p>
	 * Das sieht so aus:<br>
	 * <code>&lt;CreatedByComponent name="Erzeugt durch TIC Server Komponente" value="NMS Eingang (TIC2 Info XML)"&gt;TIC-LMS-BW/146&lt;/CreatedByComponent&gt;</code><br>
	 * Wir benutzen den Wert aus dem 'value'-Attribut.
	 * </p>
	 */
	private final TIC3Text erzeugtDurchKomponente = new TIC3Text("CreatedByComponent",
			"TIC3ErzeugtDurchTICServerKomponente", "value");

	/**
	 * Erzeugt durch System.
	 * <p>
	 * TIC3 Info XML Element: CreatedBySystem (CBS), The name of the system.
	 * </p>
	 */
	private final TIC3Text erzeugtDurchSystem = new TIC3Text("CreatedBySystem", "TIC3ErzeugtDurchSystem");

	// private TIC3Meldung tic3Meldung;

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3MeldungsDaten() {
		super(TIC3Meldung.XML_ELEMENT_MELDUNG, "");
		addAttribut(beschreibung);
		addKindElement(tic3Id);
		addKindElement(tic3Datenproduzent);
		addKindElement(tic3Datenkennung);
		addKindElement(tic3Beschreibung);
		addKindElement(tic3Location);
		addKindElement(tic3Event);
		addKindElement(dauer);
		addKindElement(nichtKodierbareInformation);
		addKindElement(prioritaet);
		addKindElement(tic3LocationGegenrichtung);
		addKindElement(name);
		addKindElement(erzeugtDurchProzess);
		addKindElement(datenerzeugungszeit);
		addKindElement(datenersterzeugungszeit);
		addKindElement(zeitZone);
		addKindElement(weitereUpdates);
		addKindElement(objektErzeugungszeit);
		addKindElement(objektEndezeit);
		addKindElement(tic3Status);
		addKindElement(speicherZeit);
		addKindElement(erzeugtDurchBenutzer);
		addKindElement(erzeugtDurchTeam);
		addKindElement(erzeugtDurchKomponente);
		addKindElement(erzeugtDurchSystem);
	}

	/**
	 * Erzeugt ein neues Objekt aus einem Knoten eines XML-Dokumentes.
	 *
	 * @param node
	 *            Knoten
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public TIC3MeldungsDaten(final Node node) throws TIC3Exception {
		this();
		if ((node.getNodeType() != Node.ELEMENT_NODE) || !node.getNodeName().equals(TIC3Meldung.XML_ELEMENT_MELDUNG)) {
			throw new TIC3Exception("falscher Knotentyp!");
		}
		einlesenXML(node);
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		super.einlesenXML(node);

		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);

			// Umleitungen
			if (n.getNodeName().equals(TIC3Umleitung.XML_ELEMENT_UMLEITUNG)) {
				final TIC3Umleitung uml = new TIC3Umleitung();
				uml.einlesenXML(n);
				umleitungen.add(uml);
			}

			// Quellen
			if (n.getNodeName().equals(TIC3Quelle.XML_ELEMENT_QUELLE)) {
				final TIC3Quelle quelle = new TIC3Quelle();
				quelle.einlesenXML(n);
				quellen.add(quelle);
			}

			// Kreuzungslokationen
			if (n.getNodeName().equals(XML_ELEMENT_CROSSLOCATIONS)) {
				final TIC3Location loc = new TIC3Location(XML_ELEMENT_CROSSLOCATIONS, null);
				loc.einlesenXML(n);
				kreuzungsLokationen.add(loc);
			}
		}
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		super.publizieren(davdata, parameter);

		// Umleitungen
		if (umleitungen.size() > 0) {
			final Data.Array array = davdata.getArray(DAV_ATTR_UMLEITUNGEN);
			array.setLength(umleitungen.size());
			for (int i = 0; i < umleitungen.size(); i++) {
				umleitungen.get(i).publizieren(array.getItem(i), parameter);
			}
		}

		// Quellen
		if (quellen.size() > 0) {
			final Data.Array array = davdata.getArray(DAV_ATTR_QUELLEN);
			array.setLength(quellen.size());
			for (int i = 0; i < quellen.size(); i++) {
				quellen.get(i).publizieren(array.getItem(i), parameter);
			}
		}

		// Kreuzungslokationen
		if (kreuzungsLokationen.size() > 0) {
			final Data.Array array = davdata.getArray(DAV_ATTR_CROSSLOCATIONS);
			array.setLength(kreuzungsLokationen.size());
			for (int i = 0; i < kreuzungsLokationen.size(); i++) {
				kreuzungsLokationen.get(i).publizieren(array.getItem(i), parameter);
			}
		}
	}

	/**
	 * Erzeugt eine PID aus den Daten der Meldung.
	 *
	 * @return PID
	 */
	@Override
	public String bildePid() {
		return "tic3.lms." + tic3Id.getText();
	}

	/**
	 * Gibt die IDNumber der Meldung zur&uuml;ck.
	 *
	 * @return IDNumber der Meldung
	 */
	public String getIDNumber() {
		return tic3Datenkennung.getText();
	}

	// /**
	// * Gibt alle in der Meldung referenzierten Locationcodes zur&uuml;ck.
	// *
	// * @return Liste von Locationcodes
	// */
	// public List<Integer> getLocationCodes() {
	// return rdsVersion.getLocationCodes();
	// }

	/**
	 * Gibt den Status der Meldungzur&uuml;ck.
	 *
	 * @return Status
	 */
	public TIC3StatusEnum getMeldungsStatus() {
		return tic3Status.getTIC3MeldungsStatus();
	}

	// /**
	// * Gibt die Versionsnummer der Meldung zur&uuml;ck.
	// *
	// * @return Versionsnummer
	// */
	// public int getVersion() {
	// return rdsVersion.verwaltungsInformationen.getVersion();
	// }
	//
	// /**
	// * Test, ob die Meldung g&uuml;ltig ist.
	// *
	// * @return true wenn g&uuml;ltig, sonst false
	// */
	// public boolean istZeitGueltig() {
	// return rdsVersion.istZeitGueltig();
	// }

	//
	// /**
	// * Setzt die Ablaufzeit der Meldung.
	// *
	// * @param neueAblaufzeit
	// * neue Ablaufzeit
	// */
	// public void setAblaufzeit(final long neueAblaufzeit) {
	// rdsVersion.verwaltungsInformationen.setAblaufzeit(neueAblaufzeit);
	// }

	/**
	 * Setzt die IDNumber der Meldung.
	 *
	 * @param nummer
	 *            IDNumber
	 */
	public void setIDNumber(final String nummer) {
		tic3Datenkennung.setText(nummer);
	}

	/**
	 * Setzt die IDOrganisation der Meldung.
	 *
	 * @param organisation
	 *            IDOrganisation der Meldung
	 */
	public void setIDOrganisation(final String organisation) {
		tic3Datenproduzent.setText(organisation);
	}

	/**
	 * Setzt den Status der Meldung.
	 *
	 * @param neuerstatus
	 *            neuer Status
	 */
	public void setMeldungsStatus(final TIC3StatusEnum neuerstatus) {
		tic3Status.setTIC3MeldungsStatus(neuerstatus);
	}

	/**
	 * Setzt die Identifikation der Meldung.
	 *
	 * @param identifikation
	 *            Identifikation
	 */
	public void setzeIdentifikation(final String identifikation) {
		tic3Id.setText(identifikation);
	}

	// It can be either 0 (positive) or 1 (negative).
	public TMCRichtungEnum getTMCRichtung(final Integer tictmcrichtung) {
		switch (tictmcrichtung) {
		case 0:
			return TMCRichtungEnum.Positiv;
		case 1:
			return TMCRichtungEnum.Negativ;
		default:
			return TMCRichtungEnum.OhneRichtung;
		}
	}

	@Override
	public String getDatenProduzent() {
		return tic3Datenproduzent.getText();
	}

	@Override
	public void setDatenProduzent(final String produzent) {
		tic3Datenproduzent.setText(produzent);
	}

	/**
	 * Gibt die Identifikation der Meldung zur&uuml;ck.
	 *
	 * @return Identifikation der Meldung
	 */
	@Override
	public String getDatenKennung() {
		return tic3Id.getText();
	}

	@Override
	public void setDatenKennung(final String kennung) {
		tic3Datenkennung.setText(kennung);
	}

	@Override
	public String getEreignisBeschreibung() {
		return tic3Beschreibung.getText();
	}

	@Override
	public void setEreignisBeschreibung(final String text) {
		tic3Beschreibung.setText(text);
	}

	@Override
	public TIC3Zeitstempel getDatenErzeugungsZeit() {
		return datenerzeugungszeit;
	}

	@Override
	public void setDatenErzeugungsZeit(final Long zeitStempel) {
		datenerzeugungszeit.setZeitstempel(zeitStempel);
	}

	@Override
	public TIC3Zeitstempel getDatenAktualisierungsZeit() {
		if (objektErzeugungszeit.hasData()) {
			return objektErzeugungszeit;
		}
		return datenerzeugungszeit;
	}

	@Override
	public void setDatenAktualisierungsZeit(final Long zeitStempel) {
		objektErzeugungszeit.setZeitstempel(zeitStempel);
	}

	@Override
	public TMCStatusEnum getTMCMeldungsStatus() {
		switch (getMeldungsStatus()) {
		case Neu:
			return TMCStatusEnum.Neu;
		case Geaendert:
			return TMCStatusEnum.Geaendert;
		case Geloescht:
			return TMCStatusEnum.Geloescht;
		default:
			return TMCStatusEnum.Unbekannt;
		}
	}

	@Override
	public void setTMCMeldungsStatus(final TMCStatusEnum stat) throws TIC3Exception {
		switch (stat) {
		case Neu:
			setMeldungsStatus(TIC3StatusEnum.Neu);
			break;
		case Geaendert:
			setMeldungsStatus(TIC3StatusEnum.Geaendert);
			break;

		case Geloescht:
			setMeldungsStatus(TIC3StatusEnum.Geloescht);
			break;
		default:
			throw new TIC3Exception("Unbekannter TMC-Meldungsstatus: " + stat);
		}
	}

	@Override
	public TIC3Zeitstempel getDatenStartZeit() {
		return datenerzeugungszeit;
	}

	@Override
	public void setDatenStartZeit(final Long zeitStempel) {
		datenerzeugungszeit.setZeitstempel(zeitStempel);
	}

	@Override
	public TIC3Zeitstempel getDatenEndeZeit() {
		return objektEndezeit;
	}

	@Override
	public void setDatenEndeZeit(final Long zeitStempel) {
		objektEndezeit.setZeitstempel(zeitStempel);
	}

	@Override
	public TMCPrioritaetEnum getTMCMeldungsPrioritaet() {
		switch (prioritaet.getTIC3MeldungsPrioritaet()) {
		case Normal:
			return TMCPrioritaetEnum.Normal;
		case Eilig:
			return TMCPrioritaetEnum.Eilig;
		case SehrEilig:
			return TMCPrioritaetEnum.SehrEilig;
		default:
			return TMCPrioritaetEnum.Unbekannt;
		}
	}

	@Override
	public void setTMCMeldungsPrioritaet(final TMCPrioritaetEnum prio) throws TIC3Exception {
		switch (prio) {
		case Normal:
			prioritaet.setTIC3MeldungsPrioritaet(TIC3PrioritaetEnum.Normal);
			break;
		case Eilig:
			prioritaet.setTIC3MeldungsPrioritaet(TIC3PrioritaetEnum.Eilig);
			break;
		case SehrEilig:
			prioritaet.setTIC3MeldungsPrioritaet(TIC3PrioritaetEnum.SehrEilig);
			break;
		default:
			throw new TIC3Exception("Unbekannte TMC-Meldungspriorität: " + prio);
		}
	}

	@Override
	public boolean istEreignisInBeidenRichtungen() {
		// GEWI, Hr. Defee, 09.08.2016:
		// Wenn OppositeDirectionLocation versorgt ist, dann ist gelten die
		// Daten für beide Richtungen
		return getTic3LocationGegenrichtung().getTic3TMCLocationen().size() > 0
				|| getTic3LocationGegenrichtung().getTic3TICLocationen().size() > 0;
	}

	@Override
	public List<TMCEvent> getTMCEreignisse() {
		final List<TMCEvent> ereignisse = new ArrayList<>();

		for (TIC3TMCEvent tmcevent : tic3Event.getTMCEreignisse()) {
			ereignisse.add(new TMCEvent(getTMCEventType(tmcevent), tmcevent.getCode()));
		}

		return ereignisse;
	}

	@Override
	public void setTMCEreignisse(final List<TMCEvent> ereignisse) throws TIC3Exception {
		for (TMCEvent e : ereignisse) {
			final TIC3TmcEventCodeTypEN t;
			switch (e.getTyp()) {
			case Ereigniscode:
				t = TIC3TmcEventCodeTypEN.Ereigniscode;
				break;
			case Empfehlungscode:
				t = TIC3TmcEventCodeTypEN.Zusatzempfehlungscode;
				break;
			default:
				throw new TIC3Exception("unbekannter Ereinistyp: " + e.getTyp());
			}

			tic3Event.addTMCEreignis(new TIC3TMCEvent(t, e.getCode()));
		}
	}

	private TMCEventType getTMCEventType(final TIC3TMCEvent tmcevent) {
		switch (tmcevent.getEventTyp()) {
		case Ereigniscode:
			return TMCEventType.Ereigniscode;
		case Zusatzempfehlungscode:
			return TMCEventType.Empfehlungscode;
		default:
			return TMCEventType.Unbekannt;
		}
	}

	public TIC3Location getTic3LocationGegenrichtung() {
		return tic3LocationGegenrichtung;
	}

	@Override
	public String getID() {
		return tic3Id.getText();
	}

	@Override
	public void setID(final String id) {
		tic3Id.setText(id);
	}

	@Override
	public List<TMCLocation> getTMCLokationen() {
		final List<TMCLocation> ergebnis = new ArrayList<>();

		final List<TIC3TMCLocation> tic3tmcLocationen = tic3Location.getTic3TMCLocationen();
		if (tic3tmcLocationen != null) {
			for (TIC3TMCLocation tic3tmcLocation : tic3tmcLocationen) {
				ergebnis.add(new TMCLocation(tic3tmcLocation.getPrimaerLokation(),
						tic3tmcLocation.getSekundaerLokation(), tic3tmcLocation.getPrimaerEntfernung(),
						tic3tmcLocation.getSekundaerEntfernung(), getTMCRichtung(tic3tmcLocation.getTmcRichtung())));
			}
		}

		return ergebnis;
	}

	@Override
	public void setTMCLokationen(final List<TMCLocation> lokationen) {
		final List<TIC3TMCLocation> tic3tmcLocationen = new ArrayList<>();

		for (TMCLocation loc : lokationen) {
			final TIC3TMCLocation tic3tmcLocation = new TIC3TMCLocation();
			tic3tmcLocation.setPrimaerLokation(loc.getPrimaerLokation());
			tic3tmcLocation.setSekundaerLokation(loc.getSekundaerLokation());
			tic3tmcLocation.setPrimaerEntfernung(loc.getPrimaerEntfernung());
			tic3tmcLocation.setSekundaerEntfernung(loc.getSekundaerEntfernung());
			tic3tmcLocationen.add(tic3tmcLocation);
		}

		tic3Location.setTic3TMCLocationen(tic3tmcLocationen);
	}

	@Override
	public List<TMCKoordinate> getKoordinaten() {
		final List<TMCKoordinate> ergebnis = new ArrayList<>();

		final List<TIC3Koordinaten> koordinaten = tic3Location.getKoordinaten();

		if (koordinaten != null) {
			for (TIC3Koordinaten tic3Koordinate : koordinaten) {
				ergebnis.add(new TMCKoordinate(tic3Koordinate.getBreite(), tic3Koordinate.getLaenge()));
			}
		}

		return ergebnis;
	}

	@Override
	public String toString() {
		return "'" + getID() + "'";
	}
}
