/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TMCInteger;

/**
 * TIC3-Fahrzeugtyp.
 * <p>Entspricht TIC3 'VehicleTypeEnum'.</p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$     
 */

public class TIC3FahrzeugTyp extends TMCInteger {
	
	/**
	 * TIC3 Info XML Element: VehicleType (VTY), The type of the vehicle such as car, truck, or any type.
	 */
	public static final String XML_ELEMENT_FAHRZEUGTYP = "VehicleType";

	/**
	 * DAV-Attribut.
	 */
	// public static final String DAV_ATTR_WEEKDAY = "Wochentage";
	
	/**
	 * Enumerator TIC3-FahrzeugTyp.
	 */
	public enum TIC3FahrzeugTypEnum {
		
		/** unbekannt. */
		Unbekannt("Unbekannt", -1),

		/** Alle.	*/
		Alle("Alle", 0),
		
		/** PKW. */
		PKW("PKW", 1),

		/** LKW. */
		LKW("LKW", 2),

		/** Bus. */
		Bus("Bus", 3), 
		
		/** SattelKfz. */
		SattelKfz("SattelKfz", 20), 
		
		/** Fahrzeug mit Anhänger. */
		FahrzeugMitAnhaenger("Fahrzeug mit Anhänger", 21), 
		
		/** LKW mit Anhänger. */
		LKWMitAnhaenger("LKW mit Anhänger", 22), 
		
		/** PKW mit Anhänger. */
		PKWMitAnhaenger("PKW mit Anhänger", 23);
		

		/**
		 * Gibt eine passende FahrzeugTyp-Instanz zur&uuml;ck.
		 *
		 * @param id
		 *            numerischer Wert
		 * @return zum numerischen Wert passende Instanz
		 * @throws TIC3Exception
		 *             bei ung&uuml;ltigem Wert
		 */
		public static TIC3FahrzeugTypEnum getInstance(final int id) throws TIC3Exception {
			switch (id) {
			case -1:
				return Unbekannt;
			case 0:
				return Alle;
			case 1:
				return PKW;
			case 2:
				return LKW;
			case 3:
				return Bus;
			case 20:
				return SattelKfz;
			case 21:
				return FahrzeugMitAnhaenger;
			case 22:
				return LKWMitAnhaenger;
			case 23:
				return PKWMitAnhaenger;
			default:
				throw new TIC3Exception("Kein TIC3-FahrzeugTyp mit id=" + id + " definiert.");
			}
		}

		/**
		 * textueller Wert.
		 */
		private final String value;

		/**
		 * numerischer Wert.
		 */
		private int id;

		/**
		 * Privater Konstruktor.
		 *
		 * @param value
		 *            textueller Wert
		 * @param id
		 *            numerischer Wert
		 */
		TIC3FahrzeugTypEnum(final String value, final int id) {
			this.value = value;
			this.id = id;
		}

		/**
		 * Gibt den numerischen Wert zur&uuml;ck.
		 *
		 * @return numerischer Wert des FahrzeugTyp
		 */
		public int getId() {
			return id;
		}

		/**
		 * Gibt den textuellen Wert zur&uuml;ck.
		 *
		 * @return textueller Wert des FahrzeugTyp
		 */
		public String getValue() {
			return value;
		}
	}

	/**
	 * FahrzeugTyp.
	 */
	private TIC3FahrzeugTypEnum typ = TIC3FahrzeugTypEnum.Unbekannt;
	
	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3FahrzeugTyp() {
		this(XML_ELEMENT_FAHRZEUGTYP, null);
	}
	
	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3FahrzeugTyp(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	@Override
	public void getDavDaten(final Data davdata) throws TIC3Exception {
		super.getDavDaten(davdata);
		typ = TIC3FahrzeugTypEnum.getInstance(getWert());
	}

	@Override
	public void getXmlDaten(final Node node) throws TIC3Exception {
		typ = TIC3FahrzeugTypEnum.getInstance(Integer.parseInt(node.getTextContent()));
	}

	@Override
	public Integer getWert() {
		return typ.getId();
	}
}
