/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TMCInteger;

/**
 * TIC3-Wochentag.
 * <p>Entspricht TIC3 'DaysOfWeekEnum'.</p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$     
 */

public class TIC3Wochentag extends TMCInteger {
	
	/**
	 * TIC3 Info XML Element: DayOfWeek (DOW), A list of days of the week such as Monday and Tuesday.
	 */
	public static final String XML_ELEMENT_WEEKDAY = "DayOfWeek";

	/**
	 * DAV-Attribut.
	 */
	public static final String DAV_ATTR_WEEKDAY = "Wochentage";
	
	/**
	 * Enumerator TIC3-Wochentag.
	 */
	public enum TIC3WochentagEnum {
		
		/** unbekannt. */
		Unbekannt("Unbekannt", -1),

		/** Montag. */
		Montag("Montag", 1),

		/** Dienstag. */
		Dienstag("Dienstag", 2),

		/** Mittwoch. */
		Mittwoch("Mittwoch", 3), 
		
		/** Donnerstag. */
		Donnerstag("Donnerstag", 4), 
		
		/** Freitag. */
		Freitag("Freitag", 5), 
		
		/** Sonnabend. */
		Sonnabend("Sonnabend", 6), 
		
		/** Sonntag. */
		Sonntag("Sonntag", 7);
		

		/**
		 * Gibt eine passende Wochentag-Instanz zur&uuml;ck.
		 *
		 * @param id
		 *            numerischer Wert
		 * @return zum numerischen Wert passende Instanz
		 * @throws TIC3Exception
		 *             bei ung&uuml;ltigem Wert
		 */
		public static TIC3WochentagEnum getInstance(final int id) throws TIC3Exception {
			switch (id) {
			case 0:
				return Unbekannt;
			case 1:
				return Montag;
			case 2:
				return Dienstag;
			case 3:
				return Mittwoch;
			case 4:
				return Donnerstag;
			case 5:
				return Freitag;
			case 6:
				return Sonnabend;
			case 7:
				return Sonntag;
			default:
				throw new TIC3Exception("Kein TIC3-Wochentag mit id=" + id + " definiert.");
			}
		}

		/**
		 * textueller Wert.
		 */
		private final String value;

		/**
		 * numerischer Wert.
		 */
		private int id;

		/**
		 * Privater Konstruktor.
		 *
		 * @param value
		 *            textueller Wert
		 * @param id
		 *            numerischer Wert
		 */
		TIC3WochentagEnum(final String value, final int id) {
			this.value = value;
			this.id = id;
		}

		/**
		 * Gibt den numerischen Wert zur&uuml;ck.
		 *
		 * @return numerischer Wert des Wochentages
		 */
		public int getId() {
			return id;
		}

		/**
		 * Gibt den textuellen Wert zur&uuml;ck.
		 *
		 * @return textueller Wert des Wochentages
		 */
		public String getValue() {
			return value;
		}
	}

	/**
	 * Wochentag.
	 */
	private TIC3WochentagEnum wochentag = TIC3WochentagEnum.Unbekannt;
	

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3Wochentag(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	@Override
	public void getDavDaten(final Data davdata) throws TIC3Exception {
		super.getDavDaten(davdata);
		wochentag = TIC3WochentagEnum.getInstance(getWert());
	}

	@Override
	public void getXmlDaten(final Node node) throws TIC3Exception {
		wochentag = TIC3WochentagEnum.getInstance(Integer.parseInt(node.getTextContent()));
	}

	@Override
	public void setDavDaten(final Data davdata) throws TIC3Exception {
		davdata.getUnscaledValue(getDavAttributName()).set(wochentag.getId());
	}
}
